# -*- coding: utf-8 -*-
"""Managers for the app models."""

from __future__ import unicode_literals

from django.conf import settings
from django.db import models
from django.db.models import Q


class PlaceQuerySet(models.QuerySet):
    def published(self):
        return self.filter(status=self.model.PUBLISHED)

    def draft(self):
        return self.filter(status=self.model.DRAFT)

    def deleted(self):
        return self.filter(status=self.model.DELETED)

    def public(self):
        return self.filter(is_public=True)

    def for_user(self, user):
        filters = Q(is_public=True) | Q(is_public=False, user=user)
        return self.filter(filters)


class PlaceManager(models.Manager):
    def get_queryset(self):
        return PlaceQuerySet(self.model, using=self._db)

    def for_site(self, site_id=None):
        if site_id is None:
            site_id = settings.SITE_ID
        return self.filter(site__id=site_id)

    def public(self, site_id=None):
        """
        Return the public places.

        By default for the current website, or for the site whose ID is passed
        as *site_id* parameter.
        """
        return self.for_site(site_id).public()

    def for_user(self, user, site_id=None):
        return self.for_site(site_id).for_user(user)

    def user_public_places(self, user, site_id=None):
        return self.for_site(site_id).public().for_user(user=user)



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-20 19:29
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('places', '0004_place_status'),
    ]

    operations = [
        migrations.AlterField(
            model_name='place',
            name='category',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='places', to='places.Category'),
        ),
        migrations.AlterField(
            model_name='place',
            name='is_public',
            field=models.BooleanField(db_index=True, default=True, verbose_name='Is public?'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-12 15:54
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('places', '0003_country_and_attributes'),
    ]

    operations = [
        migrations.AddField(
            model_name='place',
            name='status',
            field=models.CharField(choices=[('draft', 'draft'), ('published', 'published'), ('deleted', 'deleted')], db_index=True, default='draft', max_length=20),
        ),
    ]



# -*- coding: utf-8 -*-
"""Views for places app."""
from __future__ import unicode_literals

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.gis.db.models import Collect
from django.contrib.gis.db.models.functions import Distance
from django.shortcuts import get_object_or_404
from django.template.defaultfilters import slugify
from django.views.generic import ListView
from django.views.generic.detail import DetailView
from django.views.generic.edit import CreateView

from places.forms import AddPlaceForm
from places.models import Category, Place


class PlaceListView(ListView):
    """List all the public places."""

    category_slug = None

    def get_category_slug(self):
        if self.category_slug is not None:
            return self.category_slug
        else:
            return self.kwargs.get('category_slug')

    @property
    def category(self):
        print(self.get_category_slug())
        if self.get_category_slug() is not None:
            return get_object_or_404(Category, slug=self.get_category_slug())
        return None

    def get_queryset(self):
        qs = Place.objects.public().published()
        category = self.category
        if category is not None:
            qs = qs.filter(category=category)
        return qs

    def get_context_data(self, **kwargs):
        context = super(PlaceListView, self).get_context_data(**kwargs)
        context['current_category'] = self.category
        return context


class UserPlaceListView(ListView):
    """List the places for a given user."""

    def get_queryset(self):
        username = self.kwargs.get('username')
        user = get_object_or_404(User, username=username)
        if user == self.request.user:
            queryset = Place.objects.for_user(user=user)
        else:
            queryset = Place.objects.user_public_places(user=user)
        return queryset

    def get_context_data(self, **kwargs):
        context = super(UserPlaceListView, self).get_context_data(**kwargs)
        context['center'] = self.get_queryset().aggregate(position_center=Collect('position'))['position_center'].centroid
        return context


class PlaceDetailView(DetailView):
    slug_field = 'slug'

    def get_queryset(self):
        if self.request.user.is_authenticated:
            queryset = Place.objects.for_user(self.request.user)
        else:
            queryset = Place.objects.public()
        return queryset

    def get_context_data(self, **kwargs):
        context = super(PlaceDetailView, self).get_context_data(**kwargs)
        nearby_items = getattr(settings, 'PLACES_RELATED_COUNT', 5)
        try:
            # TODO: put as place method
            if self.request.user.is_authenticated:
                nearby = Place.objects.for_user(self.request.user)
            else:
                nearby = Place.objects.public()
            nearby = nearby.annotate(distance=Distance('position', self.object.position))
            nearby = nearby.exclude(id=self.object.id)
            nearby = nearby.order_by('distance')
            nearby = nearby[:nearby_items]
        except ValueError:
            nearby = Place.objects.public().order_by('?')[:nearby_items]
        context['nearby'] = nearby
        from django.core.serializers import serialize
        json = serialize('geojson',
                         Place.objects.filter(id__in=[self.object.id]),
                         geometry_field='position',
                         fields=('name',),
                         indent=4)
        context['geojson'] = json
        return context


class PlaceAddView(CreateView):
    template_name = 'places/place_form.html'

    def get_form_class(self):
        return AddPlaceForm

    def form_valid(self, form):
        form.instance.user = self.request.user
        form.instance.slug = slugify(form.instance.name)
        return super(PlaceAddView, self).form_valid(form)



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-12 15:35
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('places', '0002_picture-and-links'),
    ]

    operations = [
        migrations.AddField(
            model_name='place',
            name='country',
            field=models.CharField(blank=True, max_length=100, verbose_name='country'),
        ),
        migrations.AddField(
            model_name='place',
            name='place_attributes',
            field=django.contrib.postgres.fields.jsonb.JSONField(default=dict),
        ),
        migrations.AlterField(
            model_name='placelink',
            name='link_type',
            field=models.CharField(choices=[('website', 'Website'), ('facebook', 'Facebook'), ('twitter', 'Twitter'), ('instagram', 'Instagram'), ('pinterest', 'Pinterest')], default='website', max_length=20),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-11 09:24
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('places', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='PlaceLink',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.URLField()),
                ('link_type', models.CharField(choices=[('website', 'Website'), ('facebook', 'Facebook'), ('twitter', 'Twitter'), ('instagram', 'Instagram')], default='website', max_length=20)),
            ],
        ),
        migrations.AlterModelOptions(
            name='place',
            options={'verbose_name': 'place', 'verbose_name_plural': 'places'},
        ),
        migrations.AddField(
            model_name='place',
            name='main_picture',
            field=models.ImageField(blank=True, null=True, upload_to='places/main_picture/'),
        ),
        migrations.AddField(
            model_name='placelink',
            name='place',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='places.Place'),
        ),
    ]



# -*- coding: utf-8 -*-
"""Models for places app."""
from __future__ import unicode_literals

import datetime

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.gis.db import models
from django.contrib.postgres.fields import JSONField
from django.contrib.sites.models import Site
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from django.urls import reverse

from places.managers import PlaceManager


@python_2_unicode_compatible
class Category(models.Model):
    # site = models.ForeignKey(Site, default=settings.SITE_ID)
    name = models.CharField(_("Name"), max_length=200)
    slug = models.SlugField(_("Slug"), db_index=True)
    # private_icon
    # public_icon

    def __str__(self):
        """Category human readable name."""
        return self.name

    def get_absolute_url(self):
        return reverse('places:places-category', args=[self.slug])

    @property
    def published_places_count(self):
        return self.places.all().published().count()


@python_2_unicode_compatible
class Place(models.Model):
    DRAFT = 'draft'
    PUBLISHED = 'published'
    DELETED = 'deleted'

    STATUS_CHOICES = (
        (DRAFT, _('draft')),
        (PUBLISHED, _('published')),
        (DELETED, _('deleted')),
    )
    site = models.ForeignKey(Site,
                             default=settings.SITE_ID,
                             on_delete=models.CASCADE)
    user = models.ForeignKey(User,
                             on_delete=models.CASCADE)
    category = models.ForeignKey(Category, related_name='places',
                                 on_delete=models.CASCADE)
    name = models.CharField(_("Name"), max_length=200)
    slug = models.SlugField(_("Slug"), db_index=True)
    status = models.CharField(choices=STATUS_CHOICES, max_length=20, db_index=True,
                              default=DRAFT)

    longitude = models.FloatField(_("Longitude"), blank=True, editable=False)
    latitude = models.FloatField(_("Latitude"), blank=True, editable=False)
    position = models.PointField(_("Position"), srid=4326, blank=True, null=True)
    is_public = models.BooleanField(_("Is public?"), db_index=True, default=True)

    street = models.CharField(_("Street"), blank=True, max_length=250)
    postal_code = models.CharField(_("Postal code"), blank=True, max_length=25)
    city = models.CharField(_("City"), blank=True, max_length=100)
    country = models.CharField(_("country"), blank=True, max_length=100)

    description = models.TextField(_("Description"), blank=True)
    place_attributes = JSONField(default=dict)
    main_picture = models.ImageField(upload_to='places/main_picture/', blank=True, null=True)

    added_on = models.DateTimeField(default=datetime.datetime.now, editable=False)
    updated_on = models.DateTimeField(editable=False)

    objects = PlaceManager()

    class Meta:
        unique_together = (
            ('site', 'slug')
        )
        verbose_name = _('place')
        verbose_name_plural = _('places')

    def get_absolute_url(self):
        return reverse('places:places-detail', args=[self.slug])

    def __str__(self):
        """Human readable place name."""
        return self.name

    def save(self, *args, **kwargs):
        """Updating latitude/longitude when position is updated."""
        if self.position:
            self.latitude = self.position.x
            self.longitude = self.position.y
        self.updated_on = datetime.datetime.now()
        super(Place, self).save(*args, **kwargs)


@python_2_unicode_compatible
class PlaceLink(models.Model):
    """Associated link for a place.

    The link need to be a valid url
    """

    WEBSITE = 'website'
    FACEBOOK = 'facebook'
    TWITTER = 'twitter'
    INSTAGRAM = 'instagram'
    PINTEREST = 'pinterest'

    LINK_TYPES = (
        (WEBSITE, 'Website'),
        (FACEBOOK, 'Facebook'),
        (TWITTER, 'Twitter'),
        (INSTAGRAM, 'Instagram'),
        (PINTEREST, 'Pinterest'),
    )
    place = models.ForeignKey(Place, on_delete=models.CASCADE)
    url = models.URLField()
    link_type = models.CharField(choices=LINK_TYPES, default=WEBSITE, max_length=20)

    def __str__(self):
        """Human readable PlaceLink."""
        return "{} {}: {}".format(self.place.slug, self.link_type, self.url)



# -*- coding: utf-8 -*-
"""Places admin."""
from __future__ import unicode_literals

from django.contrib.gis import admin
from django.utils.translation import ugettext_lazy as _

from places.models import Category, Place, PlaceLink


class PlaceLinkInline(admin.TabularInline):
    model = PlaceLink


class PlaceAdmin(admin.OSMGeoAdmin):
    list_display = ('name', 'category', 'user', 'city',
                    'added_on', 'updated_on', 'status', 'is_public')
    list_filter = ('site', 'category', 'is_public', 'added_on', 'status')
    date_hierarchy = ('added_on')
    prepopulated_fields = {"slug": ("name",)}
    search_fields = ('name', 'description',)
    actions = [
        'mark_as_public', 'mark_as_private',
        'mark_as_draft', 'mark_as_published', 'mark_as_deleted'
    ]
    inlines = [PlaceLinkInline]

    def mark_as_public(self, request, queryset):
        queryset.update(is_public=True)
    mark_as_public.short_description = _("Mark as public")

    def mark_as_private(self, request, queryset):
        queryset.update(is_public=False)
    mark_as_private.short_description = _("Mark as private")

    def mark_as_draft(self, request, queryset):
        queryset.update(status=Place.DRAFT)
    mark_as_draft.short_description = _("Mark as draft")

    def mark_as_published(self, request, queryset):
        queryset.update(status=Place.PUBLISHED)
    mark_as_published.short_description = _("Mark as published")

    def mark_as_deleted(self, request, queryset):
        queryset.update(status=Place.DELETED)
    mark_as_deleted.short_description = _("Mark as deleted")


class CategoryAdmin(admin.ModelAdmin):
    list_display = ('name', 'slug')
    prepopulated_fields = {"slug": ("name",)}

admin.site.register(Place, PlaceAdmin)
admin.site.register(Category, CategoryAdmin)


@admin.register(PlaceLink)
class PlaceLinkAdmin(admin.ModelAdmin):
    list_display = ('place', 'url', 'link_type')
    list_filter = ('link_type',)
